# More miscellaneous analyses of emotion recognition; correlation between emotion vectors

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/soundgen_valid/analysis')
library(ggplot2)
library(irr)  # for Fleiss' kappa

df = read.csv('sova.csv')[, -1]
ut = droplevels(df[df$type == 'ut', ])
s = droplevels(df[df$type == 'soundgen', ])

## estimate the consistency of classification (inter-rater reliability) by split-half correlation of emotion vectors separately for human and synthetic sounds
boots = function(d, nIter = 1000) {
  time_start = proc.time()
  # set up a matrix for storing per-sound correlations
  out = matrix(levels(d$sound))
  n = nrow(out)
  rownames(out) = levels(d$sound)
  out = out[, -1]
  
  for (i in 1:nIter) {
    # take half the trials for each sound
    idx1 = idx2 = vector()
    for (j in levels(d$sound)) {
      temp = which(d$sound == j & !is.na(d$nameEmotion))
      temp1 = sample(temp, size = length(temp) / 2)
      temp2 = temp[!temp %in% temp1]
      idx1 = c(idx1, temp1)
      idx2 = c(idx2, temp2)
    }
    d1 = d[idx1, ]
    d2 = d[idx2, ]
    
    # make a table of emotion vectors per sound
    em1 = unclass(table(d1$sound, d1$nameEmotion))
    em2 = unclass(table(d2$sound, d2$nameEmotion))
    
    # get correlations between ratings on each emotion, per sound
    c = vector(length = n)
    for (j in 1:n) {
      c[j] = cor(as.numeric(em1[j, ]), as.numeric(em2[j, ])) 
    }
    out = cbind(out, c)
    if (i %% 100 == 0) {
      soundgen:::reportTime(i, nIter = nIter, time_start = time_start)
    }
  }
  
  # summarize the output by taking the mean of per-sound correlations for each iteration
  means = as.numeric(apply(out, 1, function(x) mean(as.numeric(x), na.rm = TRUE)))
  mycor = data.frame(sound = rownames(out),
                     r = means)
  mycor$callType = df$callType[match(mycor$sound, df$sound)]
  mycor$sound_num = df$sound_num[match(mycor$sound, df$sound)]
  return(mycor)
}

cor_s = boots(s, nIter = 10000)
cor_ut = boots(ut, nIter = 10000)
plot(cor_ut$r, cor_s$r)
cor(cor_ut$r, cor_s$r)

aggregate(r ~ callType, cor_s, median)
aggregate(r ~ callType, cor_ut, median)

cor_s$type = 'soundgen'
cor_ut$type = 'ut'
cor_all = rbind(cor_s, cor_ut)
boxplot(r ~ type, cor_all)

d = cor_s
d$sound = df$sound_base[match(d$sound, df$sound)]
d$delta = cor_ut$r - cor_s$r
hist(d$delta)
aggregate(delta ~ callType, d, mean)

d_plot = aggregate(delta ~ callType, d, function(x) quantile(x, probs = c(.5, .025, .975)))
d_plot = data.frame(callType = d_plot$callType,
                    fit = d_plot$delta[, 1],
                    lwr = d_plot$delta[, 2],
                    upr = d_plot$delta[, 3])

mylabels = paste0(levels(d$callType))
# capitalize the first letter
mylabels = apply(as.matrix(mylabels), 1, function(x) {
  paste0(toupper(substring(x, 1, 1)), substring(x, 2))
})

ggplot(data = d, aes(x = callType, y = delta, label = sound_num)) +
  geom_violin(fill = 'gray80', color = 'white') +
  geom_point(data = d_plot, aes(x = callType, y = fit, ymin = lwr, ymax = upr), 
             inherit.aes = FALSE, size = 3, alpha = .5) +
  # geom_errorbar(data = c_plot, aes(x = callType, y = fit, ymin = lwr, ymax = upr), inherit.aes = FALSE, width = .2) +
  geom_text(size = 3, position = position_jitter(width = .15)) + 
  scale_x_discrete(labels=mylabels) +  
  # scale_y_continuous(breaks = seq(-1, 1, by = .25)) +
  # ylim(c(-1, 1)) +
  geom_hline(yintercept=0, linetype=2) +
  ylab("Consistency of ut - soundgen") +
  xlab('') +
  theme_bw() +
  theme(axis.text.x = element_text(hjust = .5),
        panel.grid = element_blank())

# load sova_em-recogn_ver2.R
c$corCorrected = cor_ut$r - c$mycor
ggplot(data = c, aes(x = callType, y = corCorrected, label = sound_num)) +
  geom_violin(fill = 'gray80', color = 'white') +
  # geom_point(data = c_plot, aes(x = callType, y = fit, ymin = lwr, ymax = upr), 
  #            inherit.aes = FALSE, size = 3, alpha = .5) +
  # geom_errorbar(data = c_plot, aes(x = callType, y = fit, ymin = lwr, ymax = upr), inherit.aes = FALSE, width = .2) +
  geom_text(size = 3, position = position_jitter(width = .15)) + 
  scale_x_discrete(labels=mylabels) +  # % of sounds with the same top emotion for ut & soundgen
  scale_y_continuous(breaks = seq(-1, 1, by = .25)) +
  # ylim(c(-1, 1)) +
  geom_hline(yintercept=0, linetype=2) +
  ylab("s-u cor corrected for u-u cor") +
  xlab('') +
  theme_bw() +
  theme(axis.text.x = element_text(hjust = .5),
        panel.grid = element_blank())


## correlation between the full actual / permuted data for s / ut
em_s = unclass(table(s$sound_base, s$nameEmotion))
colnames(em_s) = paste0(levels(df$nameEmotion), '_s')
em_ut = unclass(table(ut$sound_base, ut$nameEmotion))
colnames(em_ut) = paste0(levels(df$nameEmotion), '_ut')

c = data.frame(sound_base = rownames(em_s), r = NA)
c$callType = df$callType[match(c$sound_base, df$sound_base)]
c$sound_num = df$sound_num[match(c$sound_base, df$sound_base)]
for (i in 1:nrow(c)) {
  ratings_s = as.numeric(em_s[i, ])
  ratings_ut = as.numeric(em_ut[i, ])
  c$r[i] = cor(ratings_s, ratings_ut) 
}

aggregate(r ~ callType, c, mean)
aggregate(r ~ callType, c, median)

# permute the dataset to estimate the level of correlation between the emotion vectors of human and synthetic sounds expected by chance
nIter = 1000

out = rep(NA, nIter)
time_start = proc.time()
for (i in 1:nIter) {
  df = read.csv('sova.csv')[, -1]
  df$sound_base = sample(df$sound_base) 
  ut = droplevels(df[df$type == 'ut', ])
  s = droplevels(df[df$type == 'soundgen', ])
  
  em_s = unclass(table(s$sound_base, s$nameEmotion))
  colnames(em_s) = paste0(levels(df$nameEmotion), '_s')
  em_ut = unclass(table(ut$sound_base, ut$nameEmotion))
  colnames(em_ut) = paste0(levels(df$nameEmotion), '_ut')
  
  c = data.frame(sound_base = rownames(em_s), r = NA)
  c$callType = df$callType[match(c$sound_base, df$sound_base)]
  c$sound_num = df$sound_num[match(c$sound_base, df$sound_base)]
  for (j in 1:nrow(c)) {
    ratings_s = as.numeric(em_s[j, ])
    ratings_ut = as.numeric(em_ut[j, ])
    c$r[j] = cor(ratings_s, ratings_ut) 
  }
  out[i] = mean(c$r)
  if (i %% 100 == 0) {
    soundgen:::reportTime(i, nIter = nIter, time_start = time_start)
  }
}
mean(out) # 0.16
hist(out)

